package drakfirsttime::firstboot;

use strict;
use lib qw(/usr/lib/libDrakX /usr/lib/libDrakX/drakfirsttime);
use standalone;
use common;
use any;
use lang;

BEGIN { unshift @::textdomains, 'drakfirstboot' }

use ugtk2 qw(:all :create);
use data;
use comdialog;

my %countries = map { lang::c2name($_) => $_ } lang::list_countries(exclude_non_installed => 1);

sub new {
    my $proto = shift;
    my $class = ref($proto) || $proto;
    my $this = bless({
		      version => "1.1",
		      drakfirstboot => N("First Boot Wizard") ,
		      mdkclub => N("Mandrakeclub Registration"),
		      pp => N("First Boot Wizard"),
		      sysconffile => '/etc/sysconfig/firstboot',
		      systemfile => '/etc/sysconfig/system'
		     }, $class);
#    my $rc = "/usr/share/themes/Galaxy/gtk-2.0/firstbootrc";
#    Gtk2::Rc->parse($rc);
    add_icon_path("/usr/share/drakfirsttime/pixmaps/");
    my $default_bg = "/usr/share/mdk/backgrounds/default.png";
    eval {
	my $root = gtkroot();
	my $gc = Gtk2::Gdk::GC->new($root);
	my $pixbuf = Gtk2::Gdk::Pixbuf->new_from_file($default_bg);
	my ($w, $h) = $root->get_size;
	my $pixmap = Gtk2::Gdk::Pixmap->new($root, $w, $h, -1);
	$pixmap->draw_pixbuf($gc, $pixbuf, 0, 0, 0, 0, -1, -1, 'max', 0, 0);
	$root->set_back_pixmap($pixmap, 0);
	$root->clear;
    };
    $this->initialize(@_);
    return;
}
sub initialize {
    my $this = shift;
    my $prog_name = shift;
    $this->{mw} = ugtk2->new($this->{$prog_name} . " " . $this->{version}, center => 1);
    gtkset_size_request($this->{mw}{window}, 600, 500);
    $this->{mw}{rwindow}->set_position('center');
    $this->{nb} = Gtk2::Notebook->new;
    $this->{Button_SkipWiz} = Gtk2::Button->new(N(" Skip Wizard ")); $this->{Button_Prev} = Gtk2::Button->new(translate("Previous")); $this->{Button_Next} = Gtk2::Button->new(translate("Next")); 
    $this->{Button_Priv} = Gtk2::Button->new(N("Privacy"));
    $this->{is_network} = network_enabled();
    $this->{is_download} = mdk_downloaded();
    my $stage = {
		 drakfirstboot => {
			0 => sub { $this->start },
			1 => sub { if ($this->{is_download}) { $this->survey } else { if ($this->{is_network}) { $this->{nb}->next_page } else { $this->startdm } } },
			2 => sub { if (!$this->{is_network}) { $this->startdm } else { $this->{nb}->next_page } },
			3 => sub { if ($this->{is_network}) { $this->mdkclub_checkuser } else { $this->{nb}->next_page } },
			4 => sub { $this->start_mdkonline }
			    },
		 mdkclub => {
			0 => sub { $this->start },
			2 => sub { $this->mdkclub_checkuser },
			3 => sub { Gtk2->main_quit }
		       },
		 pp => {
			0 => sub { $this->start },
			1 => sub { if (!$this->{is_network}) { $this->startdm } else { $this->{nb}->next_page } },
			2 => sub { $this->{is_network} and $this->mdkclub_checkuser },
			3 => sub { $this->start_mdkonline }
		       }
		};

    #- warning, there is one less page when not in download (rgs)
    if (!$this->{is_download}) { $stage->{drakfirstboot}{2} = $stage->{drakfirstboot}{3}; $stage->{drakfirstboot}{3} = $stage->{drakfirstboot}{4} }
    $this->{Button_SkipWiz}->signal_connect(clicked => sub { if ($prog_name eq 'drakfirstboot') { $this->startdm } else { Gtk2->main_quit } });
    $this->{Button_Prev}->signal_connect(clicked => sub { $this->on_prev_clicked($stage->{$prog_name}) });
    $this->{Button_Next}->signal_connect(clicked => sub { $this->on_next_clicked($stage->{$prog_name}) });
    $this->{Button_Priv}->signal_connect(clicked => sub { $this->privacy_dialog });
    my $hbox_final = gtkpack_(Gtk2::HBox->new(0, 5),
			      0, gtkpack__(create_hbox('start'),
					   $this->{Button_SkipWiz},
					   $this->{Button_Priv},
					  ),
			      1, Gtk2::Label->new(""),
			      0, gtkpack__(create_hbox('start'),
					   $this->{Button_Prev},
					   $this->{Button_Next},
					  ),
			     );

    gtkadd($this->{mw}{window},
	   gtkpack_(Gtk2::VBox->new(0, 2),
		    1, gtkset_border_width(gtkadd($this->build_ui($prog_name)), 5),
		    0, Gtk2::HSeparator->new,
		    0, gtkpack($hbox_final),
		   ),
	  );
    $this->tag_firstboot('no'); $this->{Button_Prev}->set_sensitive(0);
    $this->{Button_Next}->grab_focus;
    $this->{mw}{rwindow}->show_all;
    Gtk2->main;
    ugtk2->exit(0);
}
sub start { my $this = shift; $this->setFirstPage(0); $this->{nb}->next_page }
sub build_ui {
    my $this = shift;
    my $name = shift;
    my $prog = {
		drakfirstboot => sub { $this->drakfirsttime_wizsteps },
		mdkclub => sub { $this->mdkclub_wizsteps },
		pp => sub { $this->drakfirsttime_wizsteps }
	       };
     $prog->{$name}->();
}
sub set_clean {
    my $this = shift;
    $this->{nb}->set_show_tabs(0); $this->{nb}->set_show_border(0);
    my $l = Gtk2::Label->new($_[1]);
#    $l->set_line_wrap('TRUE');
    gtkappend_page($this->{nb},
                   gtkpack_(Gtk2::VBox->new(0, 3),
                            #0, set_darea($_[0], 'welcome'),
                            1, $l,
                           )
                  );
}
sub welcome {
    my ($this, $class) = @_;
    my $mode = 1;
    my $text = $class eq 'dl' ? N_("Hi,\n

You installed Mandrakelinux Download, the free to download and use version of Mandrakelinux.

This product represents 6 months of full-time work for Mandrakesoft engineers and the worldwide community of Mandrakelinux contributors.

At Mandrakesoft, we believe in the Open Source model and fully adhere to the GPL and everything it implies. That's why we pride ourselves in keeping this product available to all.

However, to further improve our products, we ask you to kindly spend a few minutes of your time filling out the following survey. This information will help us provide you with a product that's ever more innovating, user-friendly AND tailored to your needs.

We guarantee that your answers will be kept fully private. None of the information you provide will be communicated to third parties.

If you do not wish to answer those few questions, please click on \"Do not answer survey\".

**********
We wish to thank you for the time you spent filling out this survey.") : 
  N_("Hi,

Mandrakefirsttime is an assistant which will help you to register your pack easily in order to benefit from all services included with your Mandrakelinux pack.


Take advantage of free time-limited membership to:
     
     - Mandrakeexpert for receiving assistance
     - Mandrakeclub for joining the Mandrakelinux Community
     - Mandrakeonline for keeping your computer up-to-date


For more details about these services, consult the flyer included with your pack or visit http://www.mandrakesoft.com");
    gtkappend_page($this->{nb},
		   gtkpack_(Gtk2::VBox->new(0, 3),
			    0, set_darea(N("Welcome to Firstboot"), 'welcome'),
			    1, create_scrolled_window(gtktext_insert(Gtk2::TextView->new, translate($text))),
			    0, ( if_($class eq 'dl',
				     gtksignal_connect(gtkset_active($this->{donotsend} = Gtk2::CheckButton->new(N("Do not answer survey")), !$mode),
						       clicked => sub { $mode = !$mode; $this->{vb}->set_sensitive($mode) })
				    ))));
}
sub drakfirsttime_wizsteps {
    my $this = shift;
    if (mdk_downloaded()) {
	$this->welcome('dl');
	$this->question;
    }
    else { $this->welcome('pp');
	   #$this->set_clean(N("Welcome to Mandrakefirsttime!"), N("Mandrakefirsttime is an assistant which will help you:\n\n   - to configure the basic setup of your desktop in very few steps\n   - to register your product in order to open easily an Mandrakeclub\n     account and to benefit rapidly from all its products and services!\n\n\n"));
       }
    network_enabled() and $this->subscribe_mdkclub;
    $this->set_clean(N("Ready to go?"), N("Congratulations for choosing Mandrakelinux, the easiest and\nmost-friendly Linux distribution!\n\nWe hope it will be an enjoyable experience!\n\nIf you have any questions:\n\n   - about our products: contact us at firsttime\@mandrakesoft.com\n   - about technical support: contact us at support\@mandrakesoft.com\n"));
    $this->{nb};
}
sub mdkclub_wizsteps {
    my $this = shift;
    $this->set_clean(N("Mandrakeclub Account"), N("\nThis assistant will help you to subscribe to Mandrakeclub Services,\n"));
    network_enabled() and $this->subscribe_mdkclub;
    $this->set_clean(N("Thank You"), N("You will have more information about the club community at\nhttp://www.mandrakeclub.com.\n"));
    $this->{nb};
}
sub mdk_downloaded {
    my $isdown = get_vars_from_hash('/etc/sysconfig/system', 'META_CLASS') eq 'download' ? 1: 0;
    $isdown;
}
sub display_entry {
    my $this = shift;
    my @t;
    foreach my $l (qw(ent1 ent2 ent3 ent4 ent5)) {
	$this->{$l} = Gtk2::Entry->new_with_max_length(7);
	$this->{$l}->set_size_request(50,25);
	push(@t, $this->{$l});
	push(@t, Gtk2::Label->new("-"));
    };
    pop(@t);
    @t
}
sub get_desktop_combo {
    my $wm_optionmenu = Gtk2::OptionMenu->new;
    my @wm = get_wm_array(); unshift(@wm, N("Choose Desktop"));
    $wm_optionmenu->set_popdown_strings(@wm); 
    $wm_optionmenu;
}
sub subscribe_mdkclub {
    my $this = shift;
    $this->{product} = Gtk2::RadioButton->new_with_label(undef, N("I have a Product Key and want to register"));
    $this->{member} = Gtk2::RadioButton->new_with_label_from_widget($this->{product}, N("I am already a member"));
    $this->{nothanks} = Gtk2::RadioButton->new_with_label_from_widget($this->{product}, N("No, thanks"));
    my $nb_club = Gtk2::Notebook->new; $nb_club->set_show_tabs(0); $nb_club->set_show_border(0);
    gtkappend_page($this->{nb},
		   gtkpack_(Gtk2::VBox->new(0, 3),
			    1, create_scrolled_window(
						      #Gtk2::WrappedLabel->new(formatAlaTeX(
						      gtktext_insert(Gtk2::TextView->new,
								     N("Become a member of Mandrakeclub!

Find out all benefits and services that Mandrakeclub provides to you, such as:

   - Full access to commercial applications
   - Special download mirror list exclusively for Mandrakeclub Members
   - Voting for software to put in Mandrakelinux
   - Special discounts for products and services at Mandrakestore
   - plus much more... !!!

If you provide root password, this wizard will add a club urpmi source for commercial applications.")
								     #),
								     ),
						     [ 'never', 'automatic' ],
						     ),
			    0, gtkadd(Gtk2::Frame->new(N("Registration Type")),
				      gtkpack_(Gtk2::VBox->new(0,0),
					       map { my $t = $_->[0]; my $s = $_->[1];
						     0, gtksignal_connect($this->{$t}, clicked => sub { $nb_club->set_current_page($s) })
						 } (["product", 0], ["member", 1], ["nothanks", 2])
					      )
				     )
			   )
		  );
    gtkappend_page($this->{nb},
		   gtkpack_(Gtk2::VBox->new(0,3),
			    # 0, set_darea(N("Mandrakeclub Account"), 'club'),
			    1, $nb_club
			   )
		  );
    gtkappend_page($nb_club,
		   gtkpack_(Gtk2::VBox->new(0, 2),
			    0, gtkadd(gtkset_shadow_type($this->{gfp} = Gtk2::Frame->new(N("Product Registration")), 'etched_in'),
				      gtkpack_(Gtk2::VBox->new,
					       1, gtkset_border_width(create_packtable({ col_spacings => 10, row_spacings => 10 },
					       	        [ Gtk2::Label->new(N("Registration number:")),
					       gtkpack(Gtk2::HBox->new(0, 0),
							   $this->display_entry),
							  ],
								   [ N("Complete Name:"), $this->{name} = entry_sized() ],
								   [ N("Mail Contact:"), $this->{mailcontact} = entry_sized() ],
								   [ N("Login:"), $this->{newlogin} =  entry_sized() ],
								   [ N("Password:"), $this->{newpasswd} = entry_hidepass() ],
								   [ N("Confirm Password:"), $this->{confirm} = entry_hidepass() ],
								   #[ N("Root Password:"), $this->{newroot} = entry_hidepass() ],
								  ),
								      8),
					      )
				     ))
		  );
    gtkappend_page($nb_club,
		   gtkpack_(Gtk2::VBox->new(0, 2),
			    0, gtkadd(gtkset_shadow_type($this->{gfp} = Gtk2::Frame->new(N("Mandrakeclub Registration Update")), 'etched_in'),
				      gtkpack_(Gtk2::VBox->new(0, 5),
					       1, gtkset_border_width(create_packtable({ col_spacings => 10, row_spacings => 10 },
								   [], [],
								   [ N("Login:"), $this->{login} = Gtk2::Entry->new ],
								   [ N("Password:"), $this->{passwd} = entry_hidepass() ],
								   #[ N("Root Password:"), $this->{root} = entry_hidepass() ],
								  ),
								  	8),
					      ))
			   )
		  );
    gtkappend_page($nb_club,
		   gtkpack(Gtk2::Label->new(N("You can reuse this part of the wizard to register later by clicking\non Mandrakeonline Services on the desktop or by connecting to \nhttp://www.mandrakeclub.com.\n\nClick on Forward to Finish\n")),
	   )
   );
}
sub privacy_dialog {
    my $this = shift;
    my $d = ugtk2->new(N("Privacy Text"), grab => 1, transient => $this->{mw}{window});
    $d->{rwindow}->set_position('center_on_parent');
    gtkadd(
      $d->{window},
	   gtkpack_( Gtk2::VBox->new,
		     1, ( gtkadd(
				 gtkset_shadow_type(Gtk2::Frame->new, 'in'),
				 gtkset_size_request(create_scrolled_window(gtktext_insert(Gtk2::TextView->new, privacy_text())), 580, 440),
				)
			),      
		     0, gtkpack( create_hbox(),
				 gtksignal_connect(Gtk2::Button->new(N("Ok")), clicked => sub { Gtk2->main_quit })
			       )
		   )
	  );
    $d->{rwindow}->show_all;
    $d->main
}

sub question {
    my $this = shift;
#    my $mode = 1; my $vb;
    my %quests = (
		  a => { wg => [ N("Yes"), N("No") ], 
			 quest => N("Is this your first Linux system?") },
		  b => { wg => [ 'Windows(R)', 'MacOS(R)', 'Unix(R)' ], 
			 quest => N("What is your current operating system?") },
		  c => { wg => [ N("Yes"), N("No") ],
			 quest => N("Is this your first Mandrakelinux distribution?") },
		  d => { wg => [ 'RedHat', 'SUSE', 'Debian', N("Other") ], 
			 quest => N("What is your current operating system?") },
		  e => { wg => [ N("At each release"), N("Each time. I need to install it for professional reasons"),
				 N("At each main release (8.0, 9.0, 10.0)") ],
			 quest => N("How often do you download Mandrakelinux distributions?") },
		  f => { wg => [ N("At Home"), N("In a business"), N("Both") ],
			 quest => N("Where are you using this distribution?") },
		  g => { wg => [ N("Very small ( < 10)"), N("Small ( < 50)"), N("Medium (50 to 500)"), N("Large (> 500)") ],
		         quest => N("What is the size of your organisation?") },
		  h => { wg => [ '1', '1-5', '5-10', '10-50', N("Over 50") ],
			 quest => N("On how many computers do you plan to install it?") },
		  i => { wg => [ N("Ease of use"), N("Integrated Desktop"), N("Performance"), 
				 N("Stability"), N("Innovations"), N("Compatibility") ],
			 quest =>  N("What do you think is the most appealing feature in Mandrakelinux?") },
		  j => { wg => [ 'KDE', 'GNOME', 'IceWM', 'WindowMaker', N("Other") ], 
			 quest => N("Which Linux graphical environment do you prefer?") },
		  k => { wg => [ 'Kontact', 'Evolution', 'Mozilla Mail', N("Other") ],
			 quest => N("Which PIM (Personal Information Manager) do you prefer?") },
		  l => { wg => [ 'Konqueror', 'Mozilla', 'Mozilla Firefox', N("Other") ],
			 quest => N("Which web browser do you prefer?") },
		  n => { wg => [ N("Desktop"), N("Server"), N("Desktop and server(s)") ],
			 quest => N("For what usage do you plan to use this distribution?") },
		  o => { wg => [ N("A Workstation"), N("A Laptop"), N("Both") ],
			 quest => N("Do you use?") },
		  p => { wg => [ N("Office"), N("Multimedia"), N("Internet"), N("Development tools"), N("Server") ],
			 quest => N("Which function do you use most of the time?")
		       },
		  r => { wg => [ N("You are a faithful Mandrakelinux user"), N("You are a Linux user"), N("You want to test Linux") ],
			 quest => N("Which of the following best describes you?") },
		  t => { wg => [ N("Under 15"), '15-25', '25-35', '35-45', '45-55', N("over 55") ],
			 quest => N("How old are you?") },
		  u => { wg => [ N("Male"), N("Female") ], 
			 quest => N("Are you?") },
		  v => { wg => [ N("Yes"), N("No") ],
			 quest => N("Do you allow uploading information about your hardware settings?") },
		  w => { wg => [ N("Yes"), N("No") ], 
			 quest => N("Would you like to receive the Mandrakelinux News?") },
		 );
    my %q;
    my %sense = ( a1 => ['b', 'c'], a2 => ['c', 'b'], c1 => ['d', 'e'], c2 => ['e', 'd'], f1 => ['', 'g'], f2 => ['g', ''], f3 => [ 'g', ''], w1=> ['email', ''], w2 => ['', 'email'] );
    foreach my $st ( keys %quests ) {
	my $inval = $st . 'x';
	$q{$st . 'x'} = Gtk2::RadioButton->new_with_label(undef, '');
	$this->{$st}{val} = $inval;
	foreach my $num (1..6) {
	    if (defined $quests{$st}{wg}->[$num-1]) {
		my $val = $st . $num;
		$q{$val} = Gtk2::RadioButton->new_with_label_from_widget($q{$inval}, $quests{$st}{wg}->[$num-1]);
		$q{$val}->signal_connect(clicked => sub { 
			     $this->{$st}{val} = $val;
			         if (member($val, keys %sense)) {
				     if (defined $this->{$sense{$val}->[0]}{box}) {
					 $this->{$sense{$val}->[0]}{box}->set_sensitive(1) 
				     }
				     if (defined $this->{$sense{$val}->[1]}{box}) { 
					 $this->{$sense{$val}->[1]}{box}->set_sensitive(0);
					 $this->{$sense{$val}->[1]}{val} = $sense{$val}->[1] . 'x'
				     }
				 }
			 });
	    }
	}
	gtkpack($this->{$st}{box} = Gtk2::VBox->new(0, 0),
		gtkpack_(Gtk2::HBox->new(0,0), 0, Gtk2::Label->new($quests{$st}{quest}), 1, Gtk2::Label->new(" ")),
		create_packtable({ homogeneous => 0, col_spacings => 20, row_spacings => 0 },
				 map_index { $quests{$st}{wg}->[$::i] && [ '', $q{$st . ($::i+1)} ] } (0..5)
				));
    }
    $q{v1}->set_active(1); $this->{v}{val} = 'v1';
    gtkpack_($this->{email}{box} = Gtk2::HBox->new(0, 0),
	     0, Gtk2::Label->new(N("Email") . ':'), 1, $this->{dwnemail} = entry_sized() );
    gtkpack_($this->{country}{box} = Gtk2::HBox->new(0, 0), 
	     0, Gtk2::Label->new(N("Country") . ':'), 1, $this->{dwncountry} = Gtk2::OptionMenu->new);
    $this->{dwncountry}->set_popdown_strings(sort(keys %countries));
    $this->{dwncountry}->set_text(lang::c2name(lang::read('', $>)->{country}));
    map { defined $this->{$_->[0]}{box} and $this->{$_->[0]}{box}->set_sensitive(0) } values %sense;
    gtkappend_page($this->{nb},
		   gtkpack_(Gtk2::VBox->new(0, 0),
			    #0, set_darea(N("Users Survey"), 'question'),
			    1, gtkpack_(Gtk2::VBox->new(0, 0),
#					0, gtksignal_connect(gtkset_active($this->{donotsend} = Gtk2::CheckButton->new(N("Do not send this questionnaire")), !$mode),
	#						     clicked => sub { $mode = !$mode; $this->{vb}->set_sensitive($mode) }),
					1, gtkpack_($this->{vb} = Gtk2::VBox->new(0,0),
						    1, create_scrolled_window( gtkpack__(Gtk2::VBox->new(0,0), map { $_ ne 'm' and $this->{$_}{box} } ('a'..'w', 'email', 'country'),
											))))));
}
sub grayed {
    my $this = shift;
    my $c = $this->{mailclient}->entry->get_text ? 1 : 0;
    foreach (qw(vbox server frame)) {
	$this->{$_}->set_sensitive($c);
    }
}
sub startdm {
    my $this = shift;
    exit(0)
}
sub survey {
    my $this = shift;
    my ($resp, $send, $arch, $mail, $lang, $locale);
    my %add;
    my $netconf = '/etc/sysconfig/network-scripts/ifup.d/survey.pl';
    my $link = 'http://poll.mandrakesoft.com/online_survey.php';
    my ($r) = cat_('/etc/mandrakelinux-release') =~ /release\s(\S+)/; 
    $arch = chomp_(`/bin/arch`);
    $locale = lang::read('', $>);
    $lang = $locale->{lang};
    foreach my $m (qw(dwnemail dwncountry)) { $this->{$m} and $add{$m} = $m eq 'dwnemail' ? $this->{$m}->get_text : $this->{$m}->entry->get_text };
    map { $resp .= $this->{$_}{val} } ('a'..'w'); 
    my $ct = lc($countries{$add{dwncountry}});
    $mail = $add{dwnemail};
    if($this->{is_network} && !$this->{sent}) {
	$send = comdialog::request_URL($link . '?rep=' . $resp . '&ver=' . $r . '&arch=' . $arch . '&lang=' . $lang . '&email='. $mail . '&country_id=' . $ct);
#	print "$link $resp $r";
	$this->{sent} = 1;
    }
    if ($this->{v}{val} eq 'v1') {
	my $meta_class = get_vars_from_hash('/etc/sysconfig/system', 'META_CLASS');
	$r =~ s/\.//;
	my $db_login = 'mdk' . $r;
	system("HWDB_PASSWD=$db_login hwdb_add_system $db_login $meta_class &")
    }
    $this->{nb}->next_page;
}
sub mdkclub_checkuser {
    my $this = shift;
    $this->{productkey} = '';
    map { $this->{productkey} .= $this->{$_}->get_text } qw (ent1 ent2 ent3 ent4 ent5); 
    my $login = $this->{newlogin}->get_text || $this->{login}->get_text;
    my $link = 'http://www.mandrakeclub.com/add_user.php';
    my $subsclub = {
		    0 => sub { my $w = wait_msg($this->{mw}{window}, N("Please wait, contacting Mandrakeclub to initialize updates packages.")); $this->AddUrpmiSource(1); remove_wait_msg($w) ; $this->display_last_page },
		    1 => sub { raiseError($this->{mw}{window}, N("Product code"), N("Wrong Product Code \"%s\". Please provide a Correct Registration number or Skip the wizard \n", $this->{productkey})) },
		    2 => sub { raiseError($this->{mw}{window}, N("login in use"), N("Nickname * %s * already in use or invalid\nPlease choose another one\n", $login)) },
		    3 => sub { raiseError($this->{mw}{window}, N("Email in use"), N("Email already in use or invalid\nCould you provide another one\n")) },
		    4 => sub { raiseError($this->{mw}{window}, N("Invalid Password"), N("Password is invalid\n Please Change it")) },
		    5 => sub { raiseError($this->{mw}{window}, N("Database Error"), N("Database error at server side\n Please connect later with drakclub or drakonline wizard")) },
		    6 => sub { my $w = wait_msg(N("Please wait, contacting Mandrakeclub to initialize updates packages.")); $this->AddUrpmiSource(0); remove_wait_msg($w); $this->display_last_page },
		    7 => sub { raiseError($this->{mw}{window}, N("No account"), N("You should register to club first. Click back \nand select first choice"))}
		   };
    my $callback_club = {
			 product => sub { 
			     if ($this->{newpasswd}->get_text ne $this->{confirm}->get_text) { 
				 raiseError($this->{mw}{window}, N("Mismatch"), N("The passwords do not match\n Please try again\n"))
			     } elsif (check_valid_email($this->{mailcontact}) != 1) { 
				 raiseError($this->{mw}{window}, N("Invalid Mail"), N("Not a valid mail address!\n")) 
			     } else {
				 $this->{response} = comdialog::request_URL($link.'?Pcode=' . $this->{productkey}.'&CUSTnick=' . $this->{newlogin}->get_text . '&CUSTemail=' . $this->{mailcontact}->get_text . '&CUSTpass='.$this->{newpasswd}->get_text);
				 if ($this->{response}->is_success) {
				     my $content = $this->{response}->content;
				     if ($content =~ m/(\d+)/) { my $code = sprintf("%d",$1); eval { $subsclub->{$code}->() } }
				 } else { raiseError($this->{mw}{window}, N("Server problem"), N("Problem connecting to server \n")) }
			     }
			 },
			 member => sub { 
			     if (!$this->{login}->get_text) { raiseError($this->{mw}{window}, N("No login"), N("Login must be provided")) } else {
				 $this->{response} = comdialog::request_URL($link.'?CUSTnick='.$this->{login}->get_text . '&CUSTpass=' . $this->{passwd}->get_text);
				 if ($this->{response}->is_success) {
				     my $content = $this->{response}->content;
				     if ($content =~ m/(\d+)/) { my $code = sprintf("%d",$1); $subsclub->{$code}->() }  
				 } else { raiseError($this->{mw}{window}, N("Server problem"), N("Problem connecting to server \n")) }
			     }
			 },
			 nothanks => sub { $this->display_last_page },
			}; 
    map {
	$this->{$_}->get_active and $callback_club->{$_}->();
    } keys %$callback_club;
}
sub AddUrpmiSource {
    my $this = shift;
    my $p = shift; #notebook page 
    my ($login, $password);
    my $arch = chomp_(`/bin/arch`); $arch =~ /i386|i486|i686$/ and $arch = 'i586';
    if ($p) {
	$login = $this->{newlogin}->get_text; $password = $this->{newpasswd}->get_text;  
    } else { $login = $this->{login}->get_text; $password = $this->{passwd}->get_text; };
    my ($r) = cat_('/etc/mandrakelinux-release') =~ /release\s(\S+)/;
    eval {
	system "/usr/sbin/urpmi.addmedia", "--wget", "club.comm_$arch\_$r", "http://$login:$password\@www.mandrakeclub.com/downloads2/comm/$r/$arch/","with ./hdlist.cz";
    };
}
sub start_mdkonline {
    my $this = shift;
    -f "/usr/sbin/mdkonline" and exec("/usr/sbin/mdkonline");
    Gtk2->main_quit
}
sub wait_msg {
    my ($msg, %options) = @_;
    my $mainw = ugtk2->new('club', grab => 1, if_(exists $options{transient}, transient => $options{transient}));
    $mainw->{rwindow}->set_position($options{transient} ? 'center_on_parent' : 'center_always') if !$::isEmbedded;
    my $label = ref($msg) =~ /^Gtk/ ? $msg : Gtk2::Label->new($msg);
    gtkadd($mainw->{window}, gtkpack__(gtkadd(Gtk2::VBox->new(0, 5), $label, if_(exists $options{widgets}, @{$options{widgets}}))));
    $label->signal_connect(expose_event => sub { $mainw->{displayed} = 1; 0 });
    $mainw->sync until $mainw->{displayed};
    gtkset_mousecursor_wait($mainw->{rwindow}->window);
    $mainw->flush;
    $mainw;
}
sub remove_wait_msg { $_[0]->destroy }
sub get2final {
    my $this = shift;
    $this->{Button_Prev}->hide; $this->{Button_SkipWiz}->hide;
#    $this->{Button_Next}->set_label(N("Finish"));
}
sub display_last_page {
    my $this = shift;
    $this->get2final;
    $this->{nb}->next_page;
}
sub on_next_clicked {
    my ($this, $stage) = @_;
    $this->{Button_Prev}->set_sensitive(1);
    my $page = $this->{nb}->get_current_page;
    $page == 5 and $this->get2final;
    !$this->{is_download} && $page == 5 and $this->get2final; 
    if (exists($stage->{$page})) { $stage->{$page}->() } else { $this->{nb}->next_page };
}   
sub on_prev_clicked {
    my ($this, $stage) = @_;
    my $page = $this->{nb}->get_current_page;
    $this->{Button_Next}->get_label eq N("Finish") and $this->{Button_Next}->set_label(N("  Next  > "));
    $this->{nb}->prev_page;
    $page == 1 and $this->setFirstPage(1)
}
sub setFirstPage {
    my ($this, $state) = @_;
    if (!$state) {
	$this->{Button_Prev}->set_sensitive(1);
	$this->{Button_Priv}->hide ;	
    } else {
	$this->{Button_Prev}->set_sensitive(0);
	$this->{Button_Priv}->show
    }
}
sub tag_firstboot { 
    my ($this, $status)= @_;
    output_p $this->{sysconffile}, qq(FIRSTBOOT=$status
)
};

1;
